# Atajo Ionic Documentation


### Submitting a transaction (authentication example)
```
import { Injectable } from '@angular/core';
import { Api } from 'atajo-ionic';

@Injectable()
export class AuthSample {

  timeout: number = 10000;
  constructor(private apiProvider: Api) { }


  auth(username:string, password:string) {

      // The request data to send to your lambda
      let lambdaRequest =
        {
          credentials: {
            "username": username,
            "password": password
          }
        }

      // The name of lambda in your domain to call (e.g. maps to ./lambdas/auth.js); 
      let lambdaName = 'auth';

      this.apiProvider.addTransaction(lambdaName, lambdaRequest, {timeout:this.timeout}).subscribe(
        result => {

            console.log("TX RESULT IS : ", result); 

        },
        error => { console.error("TX ERROR IS : ", error) },
        () => { console.log("AUTH DONE"); }
      )

    });
  }


}
```



### Push Notifications

atajo-ionic ultimately uses Google's Firebase Cloud Messaging (FCM) for push notifications. This abstracts the differences between iOS and Android and provides a secondary user interface (the firebase console) for notification statistics and debugging.   

In order for a device to register, it needs to be compiled with a firebase config file : 
- google-services.json (Android)
- GoogleService-Info.plist (iOS)

These can be automatically bundled into your app using the atajo-cli (https://www.npmjs.com/package/atajo-cli)

Alternatively, you can manually add these to your app. See (https://support.google.com/firebase/answer/7015592?hl=en) for guidance.  

Once your app is compiled with this config file, you can use the atajo-ionic Push component to register the device for, and send, push notifications. 

Push notifications can be sent directly from one device to another, or from an Atajo Lambda, or through a RESTful API. 

Example component: 

```
import { Injectable } from '@angular/core';
import { Push } from 'atajo-ionic';
import { Events } from 'ionic-angular';

@Injectable()
export class PushSample {

  timeout: number = 10000;
  constructor(private push:Push, private events:Events) { }

  /*
     - Register the device for push. 
     - The DeviceId (UUID) is automatically registered.
     - You can optionally pass a user identifier (username)
     - If you passed a username, you can send push notifications using it instead of uuid.
  */
  register() {

       const username:string = 'someUserName'; 

       //Register this device (and username) for push notifications
       this.push.register(username).then(() => {
         
         //Subscribe to notifications
         this.events.subscribe('push:notification', notification => {

                if (notification.wasTapped) {
                console.log("Received in background");
                } else {
                console.log("Received in foreground");
                };
         });
        }).catch(error => {
           console.error(error);
        });

      
  }

  /*
     - Send a push notification directly from this device to another
     - The DeviceId (UUID) is automatically registered.
     - You can optionally pass a user identifier (username)
     - If you passed a username, you can send push notifications using it instead of uuid.
  */
  send() { 

      let payload = {
         "identifiers": [ "someUsername", "someUUID", ... ],     
         "title": "Test Title",
         "message": "Test Message",
         "data": { some: 'data' },
      }

      //Returns a standard Atajo Api Observable
      push.send(payload).subscribe(
            result => { },
            error => { },
            () => { }
      ); 
  
        
  }

}
```


### 



## Examples: 

### Sending a transaction to a lambda : 






## Docs

COMING SOON





## Developing (Using the module locally)

**METHOD 1**

**1. Build the module and clone package.json**
```
npm run build; node scripts/copy-package.js
```

**2. Link it out**
```
npm link; 
```

**3. Link it in**
```
cd <your-project-dir>; npm link atajo-ionic
```



**METHOD 2**

**1. Build this module**
```
npm run build; node scripts/copy-package.js
```


**2. In your ionic projects `package.json` dependencies, map the newly created `dist` directory :**
```
    "dependencies": {
       ...
        "atajo-ionic": "file:<path-to-this-repo>/atajo-ionic/dist", 
       ...
    }
```

**3. (Remove and) Install the module**
```
rm -rf node_modules/atajo-ionic; npm i
```

Repeat Steps 1 and 3 to load your changes into the test app. 


## Contributing

- Submit an Issue!
- Submit a Pull Request!
- Submit a Funny! 

