import { Injectable } from '@angular/core';
import { Http } from '../http';
var Consul = (function () {
    function Consul(http) {
        this.http = http;
    }
    Consul.prototype.getConfig = function (domain, consulAPIUrl, consulMapToken) {
        var _this = this;
        this.consulAPIUrl = consulAPIUrl;
        this.consulMapToken = consulMapToken;
        this.kvDomains = 'mobility/domains/';
        this.kvUpdates = 'mobility/update/';
        console.debug("CONSUL:GETCONFIG CALLED " + this.kvDomains + "(@ " + this.consulAPIUrl + ")");
        return new Promise(function (resolve) {
            if (!_this.consulClientToken || _this.consulClientToken == null) {
                console.debug("atajo.consul : GETTING MAP FOR ", domain);
                _this.getClientToken(domain).then(function (token) {
                    if (!token) {
                        console.warn('atajo.consul : PRODUCT SUBDOMAIN (' + domain + ') NOT CONFIGURED ON CONSUL');
                        resolve({ notFound: true });
                        return;
                    }
                    console.debug("atajo.consul : CLIENT TOKEN IS ", token);
                    _this.consulClientToken = token.toString();
                    _this.getClientConfig(domain).then(function (config) {
                        console.log('config is : ', config);
                        resolve(config);
                    }).catch(function (error) { console.error(error); resolve(false); });
                }).catch(function (error) { console.error(error); resolve(false); });
            }
            else {
                console.debug("atajo.consul : CONFIG ALREADY FETCHED");
                resolve(false);
            }
        });
    };
    Consul.prototype.getClientToken = function (domain) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.http.get(_this.consulAPIUrl + _this.kvDomains + 'map', { 'X-Consul-Token': _this.consulMapToken })
                .subscribe(function (data) {
                try {
                    var map = JSON.parse(window.atob(data[0].Value));
                    map[_this.kvDomains + domain] ? resolve(map[_this.kvDomains + domain]) : resolve(false);
                }
                catch (e) {
                    reject(e);
                }
            }, function (error) {
                reject(error);
            }, function () {
                resolve();
            });
        });
    };
    Consul.prototype.getClientConfig = function (domain) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            console.debug("atajo.consul : GETTING CLIENT CONFIG " + domain);
            _this.http.get(_this.consulAPIUrl + _this.kvDomains + domain, { 'X-Consul-Token': _this.consulClientToken })
                .subscribe(function (data) {
                try {
                    var map = JSON.parse(window.atob(data[0].Value));
                    resolve(map);
                }
                catch (e) {
                    reject(e);
                }
            }, function (error) {
                reject(error);
            }, function () {
                resolve();
            });
        });
    };
    //curl -H "X-Consul-Token:8494d9fb-943f-899e-45ef-cecb4646e9e1" https://consul.atajo.io/v1/kv/mobility/update/io.atajo.qas.netcare.assist.Ios 
    Consul.prototype.getDeploymentKeys = function (environment, updateId, consulUpdateToken) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.consulAPIUrl || !_this.kvUpdates) {
                console.warn("CONSUL NOT CONFIGURED YET. CANNOT GET DEPLOYMENT KEYS.");
                return reject("CONSUL NOT CONFIGURED YET. CANNOT GET DEPLOYMENT KEYS.");
            }
            var url = _this.consulAPIUrl + _this.kvUpdates + updateId;
            var header = { 'X-Consul-Token': consulUpdateToken };
            console.debug("atajo.consul : GETTING UPDATE DEPLOYMENT KEYS FROM " + url + " WITH HEADER : " + JSON.stringify(header));
            _this.http.get(url, header)
                .subscribe(function (data) {
                try {
                    console.log("GOT DEPLOYMENT KEYS FOR " + updateId, data);
                    var map = JSON.parse(window.atob(data[0].Value));
                    console.log("MAP IS : ", map);
                    if (map[environment + ""] && map[environment + ""].key) {
                        resolve(map[environment + ""].key);
                    }
                    else {
                        reject("Deployment key for " + environment + " not found");
                    }
                }
                catch (e) {
                    reject(e);
                }
            }, function (error) {
                reject(error);
            }, function () {
            });
        });
    };
    Consul.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Consul.ctorParameters = function () { return [
        { type: Http, },
    ]; };
    return Consul;
}());
export { Consul };
//# sourceMappingURL=index.js.map