import { Injectable } from '@angular/core';
import { SQLite } from '@ionic-native/sqlite';
var Sqlite = (function () {
    function Sqlite(sqlite) {
        this.sqlite = sqlite;
    }
    Sqlite.prototype.init = function () {
        var _this = this;
        console.log("SQLite INIT CALLED");
        this.sqlite.create({
            name: 'atajo.client.db',
            location: 'default'
        })
            .then(function (db) {
            console.log("SQLite INIT DONE");
            _this.db = db;
        })
            .catch(function (error) { return console.log("SQLite ERROR : ", error); });
    };
    Sqlite.prototype.isReady = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var interval = setInterval(function () {
                if (_this.db) {
                    resolve();
                    clearInterval(interval);
                }
                else {
                    _this.init();
                }
            }, 500);
        });
    };
    Sqlite.prototype.create = function (table, columns) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            console.log("SQLite CREATE CALLED : " + table + " -> ", columns);
            _this.isReady().then(function () {
                console.log("SQLite READY...");
                var columnsWithTypes = [];
                columns.forEach(function (column) {
                    columnsWithTypes.push(column.name + ' ' + column.type);
                });
                var query = 'CREATE TABLE IF NOT EXISTS ' + table + ' (' + columnsWithTypes.join(', ') + ')';
                console.log("SQLite EXECUTING : ", query);
                _this.db.executeSql(query, {})
                    .then(function () {
                    resolve();
                })
                    .catch(function (error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    Sqlite.prototype.set = function (table, values) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.isReady().then(function () {
                var promises = [];
                values.forEach(function (value) {
                    var rowColumns = [];
                    var rowValues = [];
                    var valuePlaceholders = [];
                    for (var v in value) {
                        if (value.hasOwnProperty(v)) {
                            rowColumns.push(v);
                            rowValues.push(JSON.stringify(value[v]));
                        }
                    }
                    rowColumns.forEach(function () {
                        valuePlaceholders.push('?');
                    });
                    var query = 'INSERT INTO ' + table + ' (' + rowColumns.join(', ') + ') VALUES (' + valuePlaceholders.join(', ') + ')';
                    console.log("SQLite EXECUTING : ", query);
                    console.log("SQLite WITH VALUES  : ", rowValues);
                    promises.push(_this.db.executeSql(query, rowValues));
                });
                Promise.all(promises).then(function () {
                    resolve();
                })
                    .catch(function (error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    Sqlite.prototype.get = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.isReady().then(function () {
                var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
                _this.db.executeSql('SELECT * FROM ' + table + ' ' + whereStatement, {})
                    .then(function (data) {
                    var rows = [];
                    var len = data.rows.length;
                    for (var i = 0; i < len; ++i) {
                        var obj = data.rows.item(i);
                        var processedRow = {};
                        for (var r in obj) {
                            if (obj.hasOwnProperty(r)) {
                                processedRow[r] = JSON.parse(obj[r]);
                            }
                        }
                        rows.push(processedRow);
                    }
                    resolve(rows);
                })
                    .catch(function (error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    Sqlite.prototype.update = function (table, values, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.isReady().then(function () {
                var setColumns = [];
                var whereColumns = [];
                var rowValues = [];
                var valuePlaceholders = [];
                values.forEach(function (value) {
                    setColumns.push(value.column + '=?');
                    rowValues.push(JSON.stringify(value.value));
                });
                if (where) {
                    where.forEach(function (value) {
                        whereColumns.push(value.column + (value.op || '=') + '?');
                        rowValues.push(JSON.stringify(value.value));
                    });
                }
                var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + whereColumns.join(', ') : '';
                var query = 'UPDATE ' + table + ' SET ' + setColumns.join(', ') + ' ' + whereStatement + '';
                console.log("UPDATE QUERY : ", query);
                _this.db.executeSql(query, rowValues).then(function (result) {
                    resolve(result);
                }).catch(function (error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    Sqlite.prototype.remove = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.isReady().then(function () {
                var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
                _this.db.executeSql('DELETE FROM ' + table + ' ' + whereStatement, {})
                    .then(function () {
                    resolve();
                })
                    .catch(function (error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    Sqlite.prototype.nuke = function (table) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.isReady().then(function () {
                _this.db.executeSql('DROP TABLE IF EXISTS ' + table, {})
                    .then(function () {
                    resolve();
                })
                    .catch(function (error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    Sqlite.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Sqlite.ctorParameters = function () { return [
        { type: SQLite, },
    ]; };
    return Sqlite;
}());
export { Sqlite };
//# sourceMappingURL=sqlite.js.map