import { Injectable } from '@angular/core';
import { FCM } from '@ionic-native/fcm';
import { Device } from '../device';
import { Events } from 'ionic-angular';
import { Api } from '../api';
var Push = (function () {
    function Push(fcm, device, events, api) {
        this.fcm = fcm;
        this.device = device;
        this.events = events;
        this.api = api;
    }
    Push.prototype.register = function (username) {
        var _this = this;
        if (username === void 0) { username = ''; }
        return new Promise(function (resolve, reject) {
            _this.fcm.getToken().then(function (token) {
                _this.events.publish('push:registration', token);
                _this.registerToken(username, token).then(function () {
                    _this.fcm.onNotification().subscribe(function (data) {
                        console.info("GOT PUSH NOTIFICATION : ", data);
                        _this.events.publish('push:notification', data);
                    });
                }).catch(reject);
            }).catch(reject);
        });
    };
    Push.prototype.send = function (payload) {
        return this.api.addTransaction('@atajo-push/send', { payload: payload }, { timeout: 30000 });
    };
    Push.prototype.registerToken = function (username, token) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.device.getDevice().then(function (device) {
                var payload = {
                    device: device,
                    token: token,
                    user: username
                };
                _this.api.addTransaction('@atajo-push/register', payload, { timeout: 30000, queue: true }).subscribe(function (result) {
                    console.log("PUSH REGISTER RESULT IS : ", result);
                    resolve(result);
                }, function (error) { console.error("PUSH REGISTER ERROR IS : ", error); reject(error); }, function () { console.log("PUSH REGISTER DONE"); });
            }).catch(function (err) {
                console.error('Push getDevice error : ', err);
                reject(err);
            });
        });
    };
    Push.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Push.ctorParameters = function () { return [
        { type: FCM, },
        { type: Device, },
        { type: Events, },
        { type: Api, },
    ]; };
    return Push;
}());
export { Push };
//# sourceMappingURL=index.js.map