
import { Injectable } from '@angular/core';
import { Http } from '../http';
import { Observable } from 'rxjs/Observable';


@Injectable()
export class Consul {

    consulAPIUrl: string;
    consulMapToken: string;
    consulUpdateToken: string;
    consulClientToken: string;
    kvDomains: string;
    kvUpdates: string;

    constructor(private http: Http) {

    }

    getConfig(domain: string, consulAPIUrl: string, consulMapToken: string): Promise<object | boolean> {


        this.consulAPIUrl = consulAPIUrl;
        this.consulMapToken = consulMapToken;
        this.kvDomains = 'mobility/domains/';
        this.kvUpdates = 'mobility/update/';


        console.debug("CONSUL:GETCONFIG CALLED " + this.kvDomains + "(@ " + this.consulAPIUrl + ")");


        return new Promise(resolve => {

            if (!this.consulClientToken || this.consulClientToken == null) {

                console.debug("atajo.consul : GETTING MAP FOR ", domain);
                this.getClientToken(domain).then(token => {

                    if (!token) {
                        console.warn('atajo.consul : PRODUCT SUBDOMAIN (' + domain + ') NOT CONFIGURED ON CONSUL');
                        resolve({ notFound: true });
                        return;
                    }


                    console.debug("atajo.consul : CLIENT TOKEN IS ", token);
                    this.consulClientToken = token.toString();
                    this.getClientConfig(domain).then(config => {

                        console.log('config is : ', config);
                        resolve(config);

                    }).catch(error => { console.error(error); resolve(false); })

                }).catch(error => { console.error(error); resolve(false); })


            }
            else {
                console.debug("atajo.consul : CONFIG ALREADY FETCHED");
                resolve(false);
            }

        });


    }

    getClientToken(domain: string) {

        return new Promise((resolve, reject) => {

            this.http.get(this.consulAPIUrl + this.kvDomains + 'map', { 'X-Consul-Token': this.consulMapToken })
                .subscribe(data => {


                    try {
                        const map: any = JSON.parse(window.atob(data[0].Value));
                        map[this.kvDomains + domain] ? resolve(map[this.kvDomains + domain]) : resolve(false);
                    }
                    catch (e) {
                        reject(e);
                    }

                }, error => {

                    reject(error);

                }, () => {

                    resolve();

                });



        })




    }

    getClientConfig(domain: string) {

        return new Promise((resolve, reject) => {

            console.debug("atajo.consul : GETTING CLIENT CONFIG " + domain);
            this.http.get(this.consulAPIUrl + this.kvDomains + domain, { 'X-Consul-Token': this.consulClientToken })
                .subscribe(data => {

                    try {
                        const map: any = JSON.parse(window.atob(data[0].Value));
                        resolve(map);
                    }
                    catch (e) {
                        reject(e);
                    }

                }, error => {

                    reject(error);

                }, () => {

                    resolve();

                });
        })
    }

    //curl -H "X-Consul-Token:8494d9fb-943f-899e-45ef-cecb4646e9e1" https://consul.atajo.io/v1/kv/mobility/update/io.atajo.qas.netcare.assist.Ios 

    getDeploymentKeys(environment: String, updateId: string, consulUpdateToken: string) {

        return new Promise((resolve, reject) => {

            if (!this.consulAPIUrl || !this.kvUpdates) {
                console.warn("CONSUL NOT CONFIGURED YET. CANNOT GET DEPLOYMENT KEYS.")
                return reject("CONSUL NOT CONFIGURED YET. CANNOT GET DEPLOYMENT KEYS.");
            }

            let url = this.consulAPIUrl + this.kvUpdates + updateId;

            let header = { 'X-Consul-Token': consulUpdateToken };
            console.debug("atajo.consul : GETTING UPDATE DEPLOYMENT KEYS FROM " + url + " WITH HEADER : " + JSON.stringify(header));

            this.http.get(url, header)
                .subscribe(data => {

                    try {
                        console.log("GOT DEPLOYMENT KEYS FOR " + updateId, data);
                        const map: any = JSON.parse(window.atob(data[0].Value));
                        console.log("MAP IS : ", map);
                        if (map[environment + ""] && map[environment + ""].key) {
                            resolve(map[environment + ""].key)
                        }
                        else {
                            reject("Deployment key for " + environment + " not found");
                        }


                    }
                    catch (e) {
                        reject(e);
                    }

                }, error => {

                    reject(error);

                }, () => {


                });
        })



    }
}

