
import { Injectable } from '@angular/core';
import { FCM } from '@ionic-native/fcm';
import { Device } from '../device';
import { Events } from 'ionic-angular';
import { Api } from '../api';
import { Observable } from "rxjs/Observable";

@Injectable()
export class Push {


    constructor(private fcm: FCM, private device: Device, private events: Events, private api: Api) { }


    public register(username: string = '') {

        return new Promise((resolve, reject) => {

            this.fcm.getToken().then(token => {

                this.events.publish('push:registration', token);

                this.registerToken(username, token).then(() => {

                    this.fcm.onNotification().subscribe(data => {

                        console.info("GOT PUSH NOTIFICATION : ", data);
                        this.events.publish('push:notification', data);

                    })

                }).catch(reject);
            }).catch(reject);

        })
    }

    public send(payload: Object) {

        return this.api.addTransaction('@atajo-push/send', { payload: payload }, { timeout: 30000 });

    }


    private registerToken(username: string, token: string) {

        return new Promise((resolve, reject) => {

            this.device.getDevice().then(device => {

                let payload = {
                    device: device,
                    token: token,
                    user: username
                }

                this.api.addTransaction('@atajo-push/register', payload, { timeout: 30000, queue:true }).subscribe(
                    result => {

                        console.log("PUSH REGISTER RESULT IS : ", result);
                        resolve(result);
                    },
                    error => { console.error("PUSH REGISTER ERROR IS : ", error); reject(error); },
                    () => { console.log("PUSH REGISTER DONE"); }
                )


            }).catch(err => {
                console.error('Push getDevice error : ', err);
                reject(err);
            })

        })


    }






}
